@@ -1,18 +1,20 @@
+import os
 import sys
-
-from setuptools_scm import integration
-from setuptools_scm.utils import do, has_command
-from setuptools_scm import git
-import pytest
+from datetime import date
 from datetime import datetime
+from datetime import timezone
 from os.path import join as opj
+from unittest.mock import Mock
+from unittest.mock import patch
+
+import pytest
+
+from setuptools_scm import git
+from setuptools_scm import integration
+from setuptools_scm import NonNormalizedVersion
 from setuptools_scm.file_finder_git import git_find_files
-
-
-skip_if_win_27 = pytest.mark.skipif(
-    sys.platform == "win32" and sys.version_info[0] < 3,
-    reason="Not supported on Windows + Python 2.7",
-)
+from setuptools_scm.utils import do
+from setuptools_scm.utils import has_command
 
 
 pytestmark = pytest.mark.skipif(
@@ -48,7 +50,7 @@ def test_root_relative_to(tmpdir, wd, monkeypatch):
     p = wd.cwd.joinpath("sub/package")
     p.mkdir(parents=True)
     p.joinpath("setup.py").write_text(
-        u"""from setuptools import setup
+        """from setuptools import setup
 setup(use_scm_version={"root": "../..",
                        "relative_to": __file__})
 """
@@ -57,6 +59,19 @@ setup(use_scm_version={"root": "../..",
     assert res == "0.1.dev0"
 
 
+def test_root_search_parent_directories(tmpdir, wd, monkeypatch):
+    monkeypatch.delenv("SETUPTOOLS_SCM_DEBUG")
+    p = wd.cwd.joinpath("sub/package")
+    p.mkdir(parents=True)
+    p.joinpath("setup.py").write_text(
+        """from setuptools import setup
+setup(use_scm_version={"search_parent_directories": True})
+"""
+    )
+    res = do((sys.executable, "setup.py", "--version"), p)
+    assert res == "0.1.dev0"
+
+
 def test_git_gone(wd, monkeypatch):
     monkeypatch.setenv("PATH", str(wd.cwd / "not-existing"))
     with pytest.raises(EnvironmentError, match="'git' was not found"):
@@ -109,6 +124,70 @@ def test_version_from_git(wd):
     wd("git tag 17.33.0-rc")
     assert wd.version == "17.33.0rc0"
 
+    # custom normalization
+    assert wd.get_version(normalize=False) == "17.33.0-rc"
+    assert wd.get_version(version_cls=NonNormalizedVersion) == "17.33.0-rc"
+    assert (
+        wd.get_version(version_cls="setuptools_scm.NonNormalizedVersion")
+        == "17.33.0-rc"
+    )
+
+
+@pytest.mark.parametrize("with_class", [False, type, str])
+def test_git_version_unnormalized_setuptools(with_class, tmpdir, wd, monkeypatch):
+    """
+    Test that when integrating with setuptools without normalization,
+    the version is not normalized in write_to files,
+    but still normalized by setuptools for the final dist metadata.
+    """
+    monkeypatch.delenv("SETUPTOOLS_SCM_DEBUG")
+    p = wd.cwd
+
+    # create a setup.py
+    dest_file = str(tmpdir.join("VERSION.txt")).replace("\\", "/")
+    if with_class is False:
+        # try normalize = False
+        setup_py = """
+from setuptools import setup
+setup(use_scm_version={'normalize': False, 'write_to': '%s'})
+"""
+    elif with_class is type:
+        # custom non-normalizing class
+        setup_py = """
+from setuptools import setup
+
+class MyVersion:
+    def __init__(self, tag_str: str):
+        self.version = tag_str
+
+    def __repr__(self):
+        return self.version
+
+setup(use_scm_version={'version_cls': MyVersion, 'write_to': '%s'})
+"""
+    elif with_class is str:
+        # non-normalizing class referenced by name
+        setup_py = """from setuptools import setup
+setup(use_scm_version={
+    'version_cls': 'setuptools_scm.NonNormalizedVersion',
+    'write_to': '%s'
+})
+"""
+
+    # finally write the setup.py file
+    p.joinpath("setup.py").write_text(setup_py % dest_file)
+
+    # do git operations and tag
+    wd.commit_testfile()
+    wd("git tag 17.33.0-rc1")
+
+    # setuptools still normalizes using packaging.Version (removing the dash)
+    res = do((sys.executable, "setup.py", "--version"), p)
+    assert res == "17.33.0rc1"
+
+    # but the version tag in the file is non-normalized (with the dash)
+    assert tmpdir.join("VERSION.txt").read() == "17.33.0-rc1"
+
 
 @pytest.mark.issue(179)
 def test_unicode_version_scheme(wd):
@@ -137,7 +216,7 @@ def test_git_dirty_notag(today, wd, monkeypatch):
     assert wd.version.startswith("0.1.dev1")
     if today:
         # the date on the tag is in UTC
-        tag = datetime.utcnow().date().strftime(".d%Y%m%d")
+        tag = datetime.now(timezone.utc).date().strftime(".d%Y%m%d")
     else:
         tag = ".d20090213"
     # we are dirty, check for the tag
@@ -145,12 +224,14 @@ def test_git_dirty_notag(today, wd, monkeypatch):
 
 
 @pytest.mark.issue(193)
-def test_git_worktree_support(wd, tmpdir):
+@pytest.mark.xfail(reason="sometimes relative path results")
+def test_git_worktree_support(wd, tmp_path):
     wd.commit_testfile()
-    worktree = tmpdir.join("work_tree")
+    worktree = tmp_path / "work_tree"
     wd("git worktree add -b work-tree %s" % worktree)
 
     res = do([sys.executable, "-m", "setuptools_scm", "ls"], cwd=worktree)
+    assert "test.txt" in res
     assert str(worktree) in res
 
 
@@ -204,7 +285,6 @@ def test_alphanumeric_tags_match(wd):
     assert wd.version.startswith("0.1.dev1+g")
 
 
-@skip_if_win_27
 def test_git_archive_export_ignore(wd, monkeypatch):
     wd.write("test1.txt", "test")
     wd.write("test2.txt", "test")
@@ -220,10 +300,9 @@ def test_git_archive_export_ignore(wd, monkeypatch):
     assert integration.find_files(".") == [opj(".", "test1.txt")]
 
 
-@skip_if_win_27
 @pytest.mark.issue(228)
 def test_git_archive_subdirectory(wd, monkeypatch):
-    wd("mkdir foobar")
+    os.mkdir(wd.cwd / "foobar")
     wd.write("foobar/test1.txt", "test")
     wd("git add foobar")
     wd.commit()
@@ -231,10 +310,9 @@ def test_git_archive_subdirectory(wd, monkeypatch):
     assert integration.find_files(".") == [opj(".", "foobar", "test1.txt")]
 
 
-@skip_if_win_27
 @pytest.mark.issue(251)
 def test_git_archive_run_from_subdirectory(wd, monkeypatch):
-    wd("mkdir foobar")
+    os.mkdir(wd.cwd / "foobar")
     wd.write("foobar/test1.txt", "test")
     wd("git add foobar")
     wd.commit()
@@ -292,9 +370,35 @@ def test_non_dotted_tag_no_version_match(wd):
 
 @pytest.mark.issue("https://github.com/pypa/setuptools_scm/issues/381")
 def test_gitdir(monkeypatch, wd):
-    """"""
+    """ """
     wd.commit_testfile()
     normal = wd.version
     # git hooks set this and break subsequent setuptools_scm unless we clean
     monkeypatch.setenv("GIT_DIR", __file__)
     assert wd.version == normal
+
+
+def test_git_getdate(wd):
+    # TODO: case coverage for git wd parse
+    today = date.today()
+
+    def parse_date():
+        return git.parse(os.fspath(wd.cwd)).node_date
+
+    git_wd = git.GitWorkdir(os.fspath(wd.cwd))
+    assert git_wd.get_head_date() is None
+    assert parse_date() == today
+
+    wd.commit_testfile()
+    assert git_wd.get_head_date() == today
+    meta = git.parse(os.fspath(wd.cwd))
+    assert meta.node_date == today
+
+
+def test_git_getdate_badgit(
+    wd,
+):
+    wd.commit_testfile()
+    git_wd = git.GitWorkdir(os.fspath(wd.cwd))
+    with patch.object(git_wd, "do_ex", Mock(return_value=("%cI", "", 0))):
+        assert git_wd.get_head_date() is None
